void drawWeatherGraph()
{
  boolean skipNextTime = false;
  buffer.strokeWeight(1.5);
  float pixelsPerMilli = float(graphWidth)/float(timestampRange);
  for (int i=1; i<weatherTimestamps.size(); i++) {
    int from = int(((float)(getWeatherTimestamp(i-1) - firstTimestamp)) * pixelsPerMilli);
    int to = int(((float)(getWeatherTimestamp(i) - firstTimestamp)) * pixelsPerMilli);
    if(showWeather) {
      buffer.stroke(36, 100, 50);
      buffer.line(from, height-getTemperature(i-1)*(float(height)/60.0), to, height-getTemperature(i)*(float(height)/60.0));
      buffer.stroke(55, 100, 40);
      buffer.line(from, height-sqrt(getSunshine(i-1))*(float(height)/100.0), to, height-sqrt(getSunshine(i))*(float(height)/100.0));
      buffer.stroke(240, 70, 70);
      buffer.line(from, height-getRainfall(i-1)*(float(height)/30.0), to, height-getRainfall(i)*(float(height)/30.0));
    }
    buffer.fill(0, 0, 50);
    Date date = new Date(getWeatherTimestamp(i));
    if ((date.getHours() % 12 == 0) && ( ! skipNextTime)) {
      // Skip next timestamp in case we get 2 readings in same hour
      skipNextTime = true;
      if(type==1) drawClockLabel(date);
      else drawLinearLabel(date,to);
    } else skipNextTime = false;
  }
}

void drawClockLabel(Date date)
{
  buffer.textAlign(CENTER,BOTTOM);
  buffer.pushMatrix();
  int dayNumber = getNumberOfDaysDiff(firstTimestamp, date.getTime());
  int totalNumberOfDays = howManyDaysOfData();
  float diameter = float(graphWidth)/float(totalNumberOfDays);
  float pos = diameter * (float(dayNumber)-0.5);
  buffer.translate(pos, height/3 - diameter/2);
  if (date.getHours() == 12) {
    if ((pos > 10) && (pos < graphWidth-10)) {
      buffer.text(pad(date.getDate()) + "/" + pad(date.getMonth()+1) + "/" + (date.getYear()-2000), 0, 0);
    }
  }
  buffer.popMatrix();
}

void drawLinearLabel(Date date, int pos)
{
  buffer.pushMatrix();
  buffer.translate(pos, 5);
  if (date.getHours() == 0) {
    if(pos < graphWidth-10) {
      buffer.rotate(HALF_PI);
      buffer.textAlign(LEFT,TOP);
      buffer.text("Midnight", 0, -textSize);
      buffer.stroke(0, 0, 20);
      buffer.line(textSize*5, -3, height, -3);
    }
  }
  if (date.getHours() == 12) {
    if (pos > 10) {
      buffer.textAlign(CENTER,TOP);
      buffer.text(pad(date.getDate()) + "/" + pad(date.getMonth()+1) + "/" + (date.getYear()-2000), 2, 0);
    }
    buffer.rotate(HALF_PI);
    buffer.textAlign(LEFT,TOP);
    buffer.text("      Noon", 0, -textSize);
  }
  buffer.popMatrix();
}

void drawCloudGraph()
{
  float spacing = 0.75;
  float cloudScaling = (height*0.75)/maxBucketFillLevel;
  cramTimestampsIntoBuckets(int(graphWidth/spacing));
  buffer.noStroke();
  for (int i=1; i<buckets.length; i++) {
    buffer.fill(0, 0, 100, 10);
    buffer.ellipse(i*spacing, height/2, buckets[i]*cloudScaling, buckets[i]*cloudScaling);
  }
}

void drawLineGraph()
{
  cramTimestampsIntoBuckets(graphWidth);
  float scaling = (height*0.75)/maxBucketFillLevel;
  buffer.strokeWeight(1);
  buffer.stroke(0, 0, 100); // White in HSB
  for (int i=1; i<buckets.length; i++) {
    buffer.line(i-1, (height-2)-(buckets[i-1]*scaling), i, (height-2)-(buckets[i]*scaling));
  }
}

void drawClockHeatMaps()
{
  buffer.noStroke();
  int pointer = 0;
  int numberOfDays = howManyDaysOfData();
  float bucketsPerDay = 130;
  float radius = (float(graphWidth)/float(numberOfDays*2));
  int thickness = height/8;
  cramTimestampsIntoBuckets(int(numberOfDays*bucketsPerDay));
  buffer.pushMatrix();
  buffer.translate(-radius, height/3);
  for (int day=0; day<numberOfDays; day++) {
    buffer.translate(radius*2, 0);
    buffer.pushMatrix();
    buffer.rotate(HALF_PI); // Rotate so that 12 noon is at the top
    for (int bucket=0; bucket<bucketsPerDay; bucket++) {
      if (colour) setHeatColour(buckets[pointer]);
      else setXrayColour(buckets[pointer]);
      buffer.rotate(2.0*PI/bucketsPerDay);
      buffer.triangle(0, 0, radius, -radius/32, radius, radius/32);
      pointer++;
    }
    buffer.popMatrix();
    // Draw the inner black circle (the hole in the donut)
    buffer.fill(0);
    buffer.noStroke();
    buffer.ellipse(0, 0, radius*2-thickness, radius*2-thickness);
    // Draw the outer black circle (in order to space rings out)
    buffer.noFill();
    buffer.stroke(0,0,0);
    buffer.strokeWeight(10);
    buffer.ellipse(0, 0, radius*2-5, radius*2-5);
  }
  buffer.popMatrix();
}

void drawLinearHeatMap()
{
  buffer.strokeWeight(1);
  cramTimestampsIntoBuckets(graphWidth);
  for (int i=0; i<buckets.length; i++) {
    if (colour) setHeatColour(buckets[i]);
    else setXrayColour(buckets[i]);
    buffer.line(i, (height/2)-(height/18), i, (height/2)+(height/18));
  }
}

void setHeatColour(int value)
{
  buffer.strokeWeight(1);
  // Hue, saturation, brightness
  buffer.stroke(239-value * heatMapContrast, 100, 20 + value*40);
  buffer.fill(239-value * heatMapContrast, 100, 20 + value*40);
}

void setXrayColour(int value)
{
  buffer.strokeWeight(1);
  // Hue, saturation, brightness
  buffer.stroke(0, 0, float(value)*heatMapContrast);
  buffer.fill(0, 0, float(value)*heatMapContrast);
}