String pad(int number) //<>//
{
  if (number <= 9) return "0" + number;
  else return "" + number;
}

void dumpRawData(String folder)
{
  Date next;
  String time = "";
  String date = "";
  long timestamp;
  String[] allVisitors = new String[visitorTimestamps.size()];
  int[][] hourlyRates = new int[howManyDaysOfData()][24];
  for (int i=0; i<visitorTimestamps.size(); i++) {
    timestamp = getVisitorTimestamp(i);
    next = new Date(timestamp);
    time = pad(next.getHours()) + ":" + pad(next.getMinutes()) + ":" + pad(next.getSeconds());
    date = pad(next.getDate()) + "," + pad(next.getMonth()+1) + "," + pad(next.getYear());
    int dayNumber = getNumberOfDaysDiff(firstTimestamp, next.getTime());
    hourlyRates[dayNumber-1][next.getHours()]++;
    // Tweak to convert from Java timestamps to Unix timestamps !!!
    next.setYear(next.getYear()-1900);
    allVisitors[i] = date + "," + time + "," + next.getTime()/1000;
  }
  String[] hourlyVisitors = flattenIntoStringList(hourlyRates);
  saveCSV(folder, allVisitors, hourlyVisitors);
  saveJSON(folder, allVisitors, hourlyVisitors);
}

String[] flattenIntoStringList(int[][] rates)
{
  long timestamp = getVisitorTimestamp(0);
  Date next;
  String date = "";
  String[] flattened = new String[rates.length * rates[0].length];
  for (int day=0; day<rates.length; day++) {
    for (int hour=0; hour<rates[0].length; hour++) {
      next = new Date(timestamp);
      date = pad(next.getDate()) + "," + pad(next.getMonth()+1) + "," + pad(next.getYear());
      flattened[(day*24) + hour] = date + "," + hour + "," + rates[day][hour];
    }
    timestamp += 24*60*60*1000;
  }
  return flattened;
}

void saveCSV(String folder, String[] allVisitors, String[] hourlyVisitors)
{
  saveStrings(folder + "all-visitors.csv", allVisitors);
  saveStrings(folder + "hourly-rates.csv", hourlyVisitors);
}

void saveJSON(String folder, String[] allVisitors, String[] hourlyVisitors)
{
  JSONObject next;
  JSONArray visitors = new JSONArray();
  for (int i=0; i<allVisitors.length; i++) {
    next = new JSONObject();
    next.setInt("day", int(allVisitors[i].split(",")[0]));
    next.setInt("month", int(allVisitors[i].split(",")[1]));
    next.setInt("year", int(allVisitors[i].split(",")[2]));
    next.setString("time", allVisitors[i].split(",")[3]);
    next.setInt("timestamp", int(allVisitors[i].split(",")[4]));
    visitors.append(next);
  }
  saveJSONArray(visitors, folder + "all-visitors.json");

  JSONArray rates = new JSONArray();
  for (int i=0; i<hourlyVisitors.length; i++) {
    next = new JSONObject();
    next.setInt("day", int(hourlyVisitors[i].split(",")[0]));
    next.setInt("month", int(hourlyVisitors[i].split(",")[1]));
    next.setInt("year", int(hourlyVisitors[i].split(",")[2]));
    next.setInt("hour", int(hourlyVisitors[i].split(",")[3]));
    next.setInt("count", int(hourlyVisitors[i].split(",")[4]));
    rates.append(next);
  }
  saveJSONArray(rates, folder + "hourly-rates.json");
}

int howManyDaysOfData()
{
  return getNumberOfDaysDiff(firstTimestamp, lastTimestamp);
}

int getNumberOfDaysDiff(long first, long second)
{
  Date firstDay = new Date(first);
  firstDay.setHours(0);
  firstDay.setMinutes(0);
  firstDay.setSeconds(0);
  Date lastDay = new Date(second);
  lastDay.setHours(23);
  lastDay.setMinutes(59);
  lastDay.setSeconds(59);
  int diff = int(lastDay.getTime() - firstDay.getTime());
  return round(float(diff)/float(1000*60*60*24));
}

long getVisitorTimestamp(int i)
{
  return ((Long)(visitorTimestamps.get(i))).longValue();
}

long getWeatherTimestamp(int i)
{
  return ((Long)(weatherTimestamps.get(i))).longValue();
}

float getTemperature(int i)
{
  return (float(temperatures.get(i).toString()));
}

float getSunshine(int i)
{
  return (float(sunshine.get(i).toString()));
}

float getRainfall(int i)
{
  return (float(rainfall.get(i).toString()));
}

void parseVisitorData(String folder)
{
  String[] years = (new File(folder)).list();
  for (int i=0; i<years.length; i++) {
    if (years[i].indexOf(".") == -1) parseMonths(folder, years[i]);
  }
  // Set first timestamp to the very beginning of the first day
  firstTimestamp = getFirstMilliInDay(getVisitorTimestamp(0));
  // Set last timestamp to the very end of the last day
  lastTimestamp = getLastMilliInDay(getVisitorTimestamp(visitorTimestamps.size()-1));
  timestampRange = int(lastTimestamp-firstTimestamp);
}

void cramTimestampsIntoBuckets(int numberOfBuckets)
{
  maxBucketFillLevel = 0;
  float pixelsPerMilli = float(numberOfBuckets)/float(timestampRange);
  buckets = new int[numberOfBuckets+2]; // A few extra at the end to stop OoB exception
  for (int i=0; i<visitorTimestamps.size(); i++) {
    int pos = int(((float)(getVisitorTimestamp(i) - firstTimestamp)) * pixelsPerMilli);
    buckets[pos]++;
    buckets[pos+1]++; // Blend and smear the values about a bit
    if (buckets[pos] > maxBucketFillLevel) maxBucketFillLevel = buckets[pos];
    if (buckets[pos+1] > maxBucketFillLevel) maxBucketFillLevel = buckets[pos+1];
  }
}

long getFirstMilliInDay(long timestamp)
{
  Date day = new Date(timestamp);
  day.setHours(0);
  day.setMinutes(0);
  day.setSeconds(0);
  return day.getTime();
}

long getLastMilliInDay(long timestamp)
{
  Date day = new Date(timestamp);
  day.setHours(23);
  day.setMinutes(59);
  day.setSeconds(59);
  return day.getTime();
}

void parseMonths(String folder, String year)
{
  String[] months = (new File(folder + year)).list();
  for (int i=0; i<months.length; i++) {
    if (months[i].indexOf(".") == -1) parseDays(folder, year, months[i]);
  }
}

void parseDays(String folder, String year, String month)
{
  String[] days = (new File(folder + year + "/" + month)).list();
  for (int i=0; i<days.length; i++) {
    if (days[i].indexOf(".") == -1) parseTimes(folder, year, month, days[i]);
  }
}

void parseTimes(String folder, String year, String month, String day)
{
  String[] times = (new File(folder + year + "/" + month + "/" + day)).list(); //<>//
  for (int i=0; i<times.length; i++) {
    if (times[i].indexOf(".TXT") != -1) {
      String[] timeChunks = times[i].split("-");
      timeChunks[2] = timeChunks[2].substring(0, 2);
      if (year.length() == 2) year = "20" + year;
      Date date = new Date(int(year), int(month)-1, int(day), int(timeChunks[0]), int(timeChunks[1]), int(timeChunks[2]));
      visitorTimestamps.add(date.getTime());
    }
  }
}

void parseWeatherData()
{
  // CSV file cut and pasted from https://www.wunderground.com/personal-weather-station/dashboard?ID=IPLYMOUT12#history/s20160728/e20160804/mweek
  String[] lines = loadStrings("weather.csv");
  if (lines != null) {
    int year = year();
    int month = 0;
    int day = 0;
    String lastGoodTemperature = "0.0";
    for (int i=0; i<lines.length; i++) {
      if (lines[i].indexOf("Temperature") != -1) {
        month = convertMonthString(lines[i]);
        day = int(lines[i].substring(4, 6));
      } else {
        int index = lines[i].indexOf(":");
        int hour = int(lines[i].substring(0, index));
        if (lines[i].split(",")[0].indexOf("pm") != -1) {
          if (hour < 12) hour += 12;
        }
        if (lines[i].split(",")[0].indexOf("am") != -1) {
          if (hour == 12) hour = 0;
        }
        int minute = int(lines[i].substring(index+1, index+3));
        Date date = new Date(year, month-1, day, hour, minute, 0);
        weatherTimestamps.add(date.getTime());
        String temp = lines[i].split(",")[1];
        if (temp.indexOf("--") != -1) temperatures.add(lastGoodTemperature);
        else {
          temperatures.add(temp.split(" ")[0]);
          lastGoodTemperature = temp.split(" ")[0];
        }
        sunshine.add(lines[i].split(",")[11].split(" ")[0]);
        rainfall.add(lines[i].split(",")[8].split(" ")[0]);
      }
    }
  }
}

int convertMonthString(String s)
{
  if (s.indexOf("Jan") == 0) return 1;
  if (s.indexOf("Feb") == 0) return 2;
  if (s.indexOf("Mar") == 0) return 3;
  if (s.indexOf("Apr") == 0) return 4;
  if (s.indexOf("May") == 0) return 5;
  if (s.indexOf("Jun") == 0) return 6;
  if (s.indexOf("Jul") == 0) return 7;
  if (s.indexOf("Aug") == 0) return 8;
  if (s.indexOf("Sep") == 0) return 9;
  if (s.indexOf("Oct") == 0) return 10;
  if (s.indexOf("Nov") == 0) return 11;
  if (s.indexOf("Dec") == 0) return 12;
  return 0;
}

String getGraphTypeName()
{
  String prefix = "";
  if (invert) prefix = prefix + "Inverted";
  if (!colour) prefix = prefix + "BW";
  if (type==0) return prefix + "LinearHeatMap";
  if (type==1) return prefix + "ClockHeatMaps";
  if (type==2) return prefix + "LineGraph";
  if (type==3) return prefix + "CloudGraph";
  return "UnknownGraphType";
}